package org.bdware.server;


import io.netty.bootstrap.ServerBootstrap;
import io.netty.channel.Channel;
import io.netty.channel.ChannelInitializer;
import io.netty.channel.EventLoopGroup;
import io.netty.channel.nio.NioEventLoopGroup;
import io.netty.channel.socket.SocketChannel;
import io.netty.channel.socket.nio.NioServerSocketChannel;
import io.netty.handler.codec.http.HttpObjectAggregator;
import io.netty.handler.codec.http.HttpServerCodec;
import io.netty.handler.stream.ChunkedWriteHandler;
import org.bdware.server.http.IndexHttpHandler;

public class IndexServer {
    final String PATH = "/Index";
    private final int port;

    private IndexServer(int port) {
        this.port = port;
    }

    public static void main(String[] args) throws Exception {
//		NodeCenterServer.start(18001);
        start(1614);

    }

    public static void start(int port) throws Exception {
        System.out.println("[IndexServer] start at:" + port);
        new IndexServer(port).start();
    }

    private void start() throws Exception {

        EventLoopGroup bossGroup = new NioEventLoopGroup(1);
        EventLoopGroup workerGroup = new NioEventLoopGroup();
        try {

            final IndexHttpHandler serverHandler = new IndexHttpHandler();
            ServerBootstrap b = new ServerBootstrap();
            b.group(bossGroup, workerGroup).channel(NioServerSocketChannel.class).localAddress(port)
                    .childHandler(new ChannelInitializer<SocketChannel>() {
                        @Override
                        protected void initChannel(SocketChannel arg0) throws Exception {
                            arg0.pipeline().addLast(new HttpServerCodec()).addLast(new HttpObjectAggregator(65536))
                                    .addLast(new ChunkedWriteHandler()).addLast(serverHandler);
                        }
                    });
            Channel ch = b.bind(port).sync().channel();
            ch.closeFuture().sync();
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            bossGroup.shutdownGracefully();
            workerGroup.shutdownGracefully();
        }
    }
}
