package org.bdware.server.test;

import com.google.protobuf.ByteString;
import org.bdware.bdledger.api.grpc.Client;
import org.bdware.bdledger.api.grpc.pb.CommonProto;
import org.bdware.bdledger.api.grpc.pb.LedgerProto;
import org.bdware.bdledger.api.grpc.pb.QueryProto;
import org.junit.Before;
import org.junit.Test;

import java.nio.charset.StandardCharsets;
import java.security.SecureRandom;

public class BDLedgerClient {
    private Client client;

    @Before
    public void initClient() {
        //
        client = new Client("39.104.201.40", 21121);
        System.out.println(client.getLedgersSync().getLedgersList());
    }

    public static String byteArray2Str(byte[] byteArray, int offset) {
        StringBuilder sb = new StringBuilder();
        for (int j = offset; j < byteArray.length; j++) {
            byte b = byteArray[j];
            int i = ((int) b) & 0xff;
            sb.append(String.format("%x%x", i >> 4, i & 0xf));
        }
        return sb.toString();
    }

    @Test
    public void createLedger() {
        client.createLedger("default");
    }

    static SecureRandom random = new SecureRandom((System.currentTimeMillis() + "").getBytes());

    @Test
    public void send() {
        String mockedFrom = "0xb60e8dd61c5d32be8058bb8eb970870f07233155";

        LedgerProto.SendTransactionResponse ret2 =
                client.sendTransactionSync(
                        "default",
                        CommonProto.TransactionType.MESSAGE,
                        mockedFrom,
                        random.nextLong(),
                        mockedFrom,
                        "Hellooooo".getBytes(StandardCharsets.UTF_8));
        ByteString hash = ret2.getHash();
        System.out.println("[BDLedgerClient] hash:" + byteArray2Str(hash.toByteArray(), 0));
    }

    @Test
    public void countBlock() {
        // http://069.node.internetapi.cn:21030/SCIDE/CMManager?action=queryDataByHash&hash=30f0b38a845bc3280f0bafa93b2034c8bc494637
        // "ea508a07b79afc03c94a84ff190ca29f1153ef75"
        // cb304919522a4acd8f2b23fadf993829ac40795a
        QueryProto.GetBlocksResponse blocks =
                client.getBlocksSync(
                        "default",
                        System.currentTimeMillis() - 23L * 3600L * 1000L,
                        QueryProto.IncludeTransactions.FULL);
        System.out.println("BlockCount:" + blocks.getBlocksCount());
        // GetTransactionByHashResponse result = client.getTransactionByHashSync("test",
        // "30f0b38a845bc3280f0bafa93b2034c8bc494637");
        // ByteString bytes = result.getTransaction().getData();
        // System.out.println(new String(bytes.toByteArray()));
    }

    @Test
    public void queryTransaction() {
        // test2       31b50daa8d607c673af5ef449a4d78c70bf952d4
        // bdcontract  cb304919522a4acd8f2b23fadf993829ac40795a
        QueryProto.GetTransactionByHashResponse result =
                client.getTransactionByHashSync(
                        "default", "78bf9fb27963b26bf2f8d558f20bf44559178b67");
        ByteString bytes = result.getTransaction().getData();
        System.out.println(new String(bytes.toByteArray()));
    }
}
